"""
Created on 21 Mar 2019

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

https://www.pythonforbeginners.com/modules-in-python/python-secure-ftp-module
https://github.com/paramiko/paramiko/issues/1369
"""

import pysftp


# --------------------------------------------------------------------------------------------------------------------

class SFTPClient(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host):
        """
        Constructor
        """
        self.__host = host
        self.__conn = None


    # ----------------------------------------------------------------------------------------------------------------

    def connect(self, username=None, private_key=None, password=None, port=22, private_key_pass=None, ciphers=None,
                cnopts=None, default_path=None):

        # cnopts = pysftp.CnOpts()
        # cnopts.log = True

        self.__conn = pysftp.Connection(self.__host, username=username, private_key=private_key, password=password,
                                        port=port, private_key_pass=private_key_pass, ciphers=ciphers,
                                        cnopts=cnopts, default_path=default_path)


    def close(self):
        if self.__conn:
            self.__conn.close()
            self.__conn = None

    # ----------------------------------------------------------------------------------------------------------------

    def chdir(self, remote_path):
        if not self.__conn:
            raise RuntimeError('no connection')

        self.__conn.chdir(remote_path)


    def cwd(self):
        if not self.__conn:
            raise RuntimeError('no connection')

        return self.__conn.getcwd()


    def exists(self, remote_path):
        if not self.__conn:
            raise RuntimeError('no connection')

        return self.__conn.exists(remote_path)


    def get(self, remote_path, local_path=None, callback=None):
        if not self.__conn:
            raise RuntimeError('no connection')

        self.__conn.get(remote_path, local_path, callback=callback)


    def listdir(self, remote_path='.'):
        if not self.__conn:
            raise RuntimeError('no connection')

        return self.__conn.listdir(remote_path)


    def mkdir(self, remote_path):
        if not self.__conn:
            raise RuntimeError('no connection')

        self.__conn.mkdir(remote_path)


    def put(self, local_path, remote_path=None, callback=None, confirm=True):
        if not self.__conn:
            raise RuntimeError('no connection')

        return self.__conn.put(local_path, remote_path, callback=callback, confirm=confirm)


    def rmdir(self, remote_path):
        if not self.__conn:
            raise RuntimeError('no connection')

        self.__conn.rmdir(remote_path)


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        cwd = None if self.__conn is None else self.cwd()

        return "SFTPClient:{host:%s, cwd:%s}" % (self.__host, cwd)



"""
Created on 20 Nov 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

http://stackoverflow.com/questions/4271740/how-can-i-use-python-to-get-the-system-hostname
http://code.activestate.com/recipes/577972-disk-usage/
"""

import os
import socket

from pathlib import Path

from scs_core.estate.git_pull import GitPull

from scs_core.sys.ipv4_address import IPv4Address
from scs_core.sys.node import Node
from scs_core.sys.persistence_manager import FilesystemPersistenceManager


# --------------------------------------------------------------------------------------------------------------------

class Host(Node, FilesystemPersistenceManager):
    """
    Darwin, Linux or Windows 10
    """

    # ----------------------------------------------------------------------------------------------------------------
    # directories and files...

    OS_ENV_PATH =           'SCS_ROOT_PATH'

    __SCS_DIR =             "SCS"                               # hard-coded rel path
    __LATEST_UPDATE =       "latest_update.txt"                 # hard-coded rel path


    # ----------------------------------------------------------------------------------------------------------------
    # host acting as DHCP server...

    __SERVER_IPV4_ADDRESS = "192.168.2.1"                       # had-coded abs path - was None


    # ----------------------------------------------------------------------------------------------------------------
    # network identity...

    @classmethod
    def name(cls):
        full_name = socket.gethostname()
        full_names = full_name.split('.')

        return full_names[0]


    @classmethod
    def server_ipv4_address(cls):
        return IPv4Address.construct(cls.__SERVER_IPV4_ADDRESS)


    # ----------------------------------------------------------------------------------------------------------------
    # software update...

    @classmethod
    def software_update_report(cls):
        git_pull = GitPull.load(cls)

        return None if git_pull is None else str(git_pull.pulled_on.datetime.date())


    # ----------------------------------------------------------------------------------------------------------------
    # filesystem paths...

    @classmethod
    def home_path(cls):
        return os.environ[cls.OS_ENV_PATH] if cls.OS_ENV_PATH in os.environ else str(Path.home())


    @classmethod
    def scs_path(cls):
        return os.path.join(cls.home_path(), cls.__SCS_DIR)



"""
Created on 26 May 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

A Unix domain socket abstraction, implementing ProcessComms

https://pymotw.com/2/socket/uds.html
https://gist.github.com/BenKnisley/5647884
"""

import os
import socket
import time

from scs_core.sys.process_comms import ProcessComms


# --------------------------------------------------------------------------------------------------------------------

class DomainSocket(ProcessComms):
    """
    classdocs
    """

    EOM = '\n'                              # end of message for client-server communications

    __PERMISSIONS = 0o666                   # srw-rw-rw-
    __BACKLOG = 1                           # number of unaccepted connections before refusing new connections
    __BUFFER_SIZE = 1024
    __WAIT_FOR_AVAILABILITY = 1.0           # seconds

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def __read(cls, connection):
        message = b''

        while True:
            data = connection.recv(cls.__BUFFER_SIZE)

            if not data:
                break

            message += data

        return message.decode()


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, path):
        """
        Constructor
        """
        self.__path = path                  # string

        self.__socket = None                # socket.socket
        self.__conn = None                  # (socket object, address info)


    # ----------------------------------------------------------------------------------------------------------------

    def connect(self, wait_for_availability=True):
        self.__socket = socket.socket(family=socket.AF_UNIX, type=socket.SOCK_STREAM)


    def accept(self):
        self.__socket.bind(self.path)
        self.__socket.listen(self.__BACKLOG)

        os.chmod(self.path, self.__PERMISSIONS)

        self.__conn, _ = self.__socket.accept()


    def close(self):
        if self.__socket:
            self.__socket.close()


    # ----------------------------------------------------------------------------------------------------------------
    # client-server API...

    def server_send(self, message):
        self.__send(self.__conn, message)


    def client_send(self, message):
        try:
            self.__socket.connect(self.path)
        except OSError:
            pass                    # assume that the socket is already connected

        self.__send(self.__socket, message)


    def server_receive(self):
        return self.__receive(self.__conn)


    def client_receive(self):
        return self.__receive(self.__socket)


    # ----------------------------------------------------------------------------------------------------------------

    def __send(self, channel, message):
        channel.send((message + self.EOM).encode())


    def __receive(self, channel):
        message = ''
        while True:
            char = channel.recv(1).decode()

            if char == self.EOM:
                return message

            if len(message) == self.__BUFFER_SIZE:
                raise ValueError(message)

            message += char


    # ----------------------------------------------------------------------------------------------------------------
    # unidirectional API...

    def read(self):                                             # blocking
        # socket...
        self.__socket.bind(self.path)
        self.__socket.listen(self.__BACKLOG)

        os.chmod(self.path, self.__PERMISSIONS)

        try:
            while True:
                self.__conn, _ = self.__socket.accept()

                try:
                    # data...
                    yield DomainSocket.__read(self.__conn).strip()

                finally:
                    self.__conn.close()

        finally:
            os.unlink(self.path)


    def write(self, message, wait_for_availability=True):       # message is dispatched on close()
        # socket...
        while True:
            try:
                self.__socket.connect(self.path)
                break

            except (socket.error, FileNotFoundError) as ex:
                if not wait_for_availability:
                    raise ConnectionRefusedError(ex)

                time.sleep(self.__WAIT_FOR_AVAILABILITY)

        # data...
        self.__socket.sendall(message.strip().encode())


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def path(self):
        return self.__path


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "posix.DomainSocket:{path:%s, socket:%s}" % (self.path, self.__socket)



#!/usr/bin/env python3

"""
Created on 4 Sep 2020
Updated 23 Mar 2021

@author: Jade Page (jade.page@southcoastscience.com)

https://packaging.python.org/tutorials/packaging-projects/
https://packaging.python.org/guides/single-sourcing-package-version/
"""

import codecs
import os
import setuptools


# --------------------------------------------------------------------------------------------------------------------

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path)) as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            return line.split("'")[1]
    else:
        raise RuntimeError("Unable to find version string.")


# --------------------------------------------------------------------------------------------------------------------

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as req_txt:
    required = [line for line in req_txt.read().splitlines() if line]


setuptools.setup(
    name="scs-host-posix",
    version=get_version("src/scs_host/__init__.py"),
    author="South Coast Science",
    author_email="contact@southcoastscience.com",
    description="Host abstractions for data consumers running POSIX-compliant operating systems, including Windows 10.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/south-coast-science/scs_host_posix",
    package_dir={'': 'src'},
    packages=setuptools.find_packages('src'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: POSIX",
    ],
    python_requires='>3.6',
    install_requires=required,
    platforms=['any'],
)
